/*
	Netvideo version 3.1
	Written by Ron Frederick <frederick@parc.xerox.com>

	Simple hack to translate a Tcl/Tk init file into a C string constant
*/

/*
 * Copyright (c) Xerox Corporation 1992. All rights reserved.
 *  
 * License is granted to copy, to use, and to make and to use derivative
 * works for research and evaluation purposes, provided that Xerox is
 * acknowledged in all documentation pertaining to any such copy or derivative
 * work. Xerox grants no other licenses expressed or implied. The Xerox trade
 * name should not be used in any advertising without its written permission.
 *  
 * XEROX CORPORATION MAKES NO REPRESENTATIONS CONCERNING EITHER THE
 * MERCHANTABILITY OF THIS SOFTWARE OR THE SUITABILITY OF THIS SOFTWARE
 * FOR ANY PARTICULAR PURPOSE.  The software is provided "as is" without
 * express or implied warranty of any kind.
 *  
 * These notices must be retained in any copies of any part of this software.
 */

#include <stdio.h>
#include <string.h> /* strcasecmp() */
#include <ctype.h>

#ifdef _WIN32
#define strcasecmp _stricmp
#endif

/*
 * Define TCL2C_INT if your compiler has problems with long strings.
 */
#if defined(WIN32) || defined(__alpha__) || defined(__hpux)
#define TCL2C_INT
#endif


/*XXX*/
void put(int c)
{
#ifdef TCL2C_INT
	static int n;

	if ((++n % 20) == 0)
		printf("%u,\n", c);
	else
		printf("%u,", c);
	/* printf("%u,%c", c, ((++n & 0xf) == 0) ? '\n' : ' '); */
#else
	switch(c) {
	case '\\':
	case '"':
		putchar('\\');
		break;
	case '\n':
		printf("\\n\\");
		break;
	default:
		break;
	}
	printf("%c", c);
#endif
}

void onefile(FILE* f, int skip_source)
{
	int nl = 1;
	int skipping = 0;
	int c;

	while ((c = getc(f)) != EOF) {
		switch (c) {
		case ' ':
		case '\t':
		case '\f':
			if (nl || skipping)
				continue;
			break;
		case '#':
			if (skipping)
				continue;
			if (nl) {
				skipping = 1;
				continue;
			}
			break;
		case '\n':
			if (skipping) {
				skipping = 0;
				nl = 1;
				continue;
			}
			nl = 1;
			break;
		case '\\':
			if (skipping) {
				c = getc(f);
				continue;
			}
			break;
		case 's':
			if (skipping)
				continue;
			if (nl && skip_source) {
				/* skip 'source' lines */
				const char* targ = "source";
				const char* cp = targ + 1;
				while ((c = getc(f)) == *cp)
					++cp;
				if (*cp == 0) {
					if (c == ' ' || c == '\t') {
						skipping = 1;
						continue;
					}
				}
				for ( ; targ < cp; ++targ)
					put(*targ);
			}
			nl = 0;
			break;
		default:
			nl = 0;
			if (skipping)
				continue;
			break;
		}
		put(c);
	}
}

int main(int argc, char **argv)
{
	int skip_source;
	const char* name;
	
	if (argc < 2) {
		fprintf(stderr, "Usage: tcl2c++ name [ file ... ]\n");
		exit(1);
	}
#ifdef TCL2C_INT
	printf("static const char code[] = {\n");
#else
	printf("static const char code[] = \"");
#endif
	name = argv[1];

	/*XXX tk hack */
	skip_source = (strcasecmp(name, "et_tk") == 0) ? 1 : 0;

	if (argc == 2)
		onefile(stdin, skip_source);
	else {
		for ( ; --argc >= 2; ++argv) {
			const char* filename = argv[2];
			FILE* f = fopen(filename, "r");
			if (f == 0) {
				perror(filename);
				exit(1);
			}
			onefile(f, skip_source);
			fclose(f);
		}
	}
#ifdef TCL2C_INT
	printf("0 };\n");
#else
	printf("\";\n");
#endif
	printf("#include \"tclcl.h\"\n");
	printf("EmbeddedTcl %s(code);\n", name);
	return (0);
}
